#include "ai.h"

// Correspond to the center spot on the tic-tac-toe board
#define CENTER_X 2
#define CENTER_Y 2

//		X|O|X		5 char's wide
//		-----
//		O|O|X
//		-----
//		O|X|O

//		5 char's tall

// Moves the AI in the pattern we pre-determined would suit the AI best
void AIMove(GAMEDATA &gamedata)
{
	if(Move2Win(gamedata))
		return;
	else if(Move2Block(gamedata))
		return;
	else if(Move2CenterOrCorner(gamedata))
		return;
	else
		Move2OpenSq(gamedata);
}

// This is going to see if we can move the AI to a spot and win
bool Move2Win(GAMEDATA &gamedata)
{
	// Notice how we skip by 2 -- This is so we are ONLY checking valid tic-tac-toe
	// spots -- Look at the tic-tac-toe board at the top of this file for reference
	for(int y = 0; y < BOARD_WDTH; y += 2)
	{
		for(int x = 0; x < BOARD_HGT; x += 2)
		{
			// If the square is open
			if(gamedata.squareOpen(x,y,false))
			{
				// Put a 'O' there
				gamedata.setScreenBuff(x,y,'O',false);

				// If we win -- Return true
				if(gamedata.threeHorz('O') || gamedata.threeVert('O') ||
				   gamedata.twoDiag('O'))
						return true;

				// Otherwise put it back the way it was
				gamedata.setScreenBuff(x,y,OPEN,false);
			}

		}

	}

	return false;
}

// This tests to see if we can block the player from winning next turn
bool Move2Block(GAMEDATA &gamedata)
{
	// Again we skip by two so we only check valid tic-tac-toe squares
	for(int y = 0; y < BOARD_WDTH; y += 2)
	{
		for(int x = 0; x < BOARD_HGT; x += 2)
		{
			// If the square is open
			if(gamedata.squareOpen(x,y,false))
			{
				// Temporarily move the opponent there
				gamedata.setScreenBuff(x,y,'X',false);

				// If the opponent would win as a result
				if(gamedata.threeHorz('X') || gamedata.threeVert('X') ||
				   gamedata.twoDiag('X'))
				{
					// Block that spot
					gamedata.setScreenBuff(x,y,'O',false);
						return true;
				}
				else
					gamedata.setScreenBuff(x,y,OPEN,false); // Otherwise put it back to "open"
			}

		}

	}

	return false;

}

// If we can move to the center spot do so -- Otherwise if we can move to a corner (starting
// at the top left and working clockwise) do so -- Otherwise return false
bool Move2CenterOrCorner(GAMEDATA &gamedata)
{
	// Move to the center if possible
	if(gamedata.squareOpen(CENTER_X,CENTER_Y,false))
	{
		gamedata.setScreenBuff(CENTER_X,CENTER_Y,'O',false);
			return true;
	}

	// Move to a corner if possible -- Notice how we skip 4 this time --
	// Check out the board at the top of this file if that's confusing
	for(int y = 0; y < BOARD_WDTH; y += 4)
	{
		for(int x = 0; x < BOARD_HGT; x += 4)
		{
			if(gamedata.squareOpen(x,y,false))
			{
				gamedata.setScreenBuff(x,y,'O',false);
					return true;
			}
		}
	}

	return false;
}

// Just move to the first open square you come across
void Move2OpenSq(GAMEDATA &gamedata)
{
	for(int y = 0; y < BOARD_WDTH; y += 2)
	{
		for(int x = 0; x < BOARD_HGT; x += 2)
		{
			if(gamedata.squareOpen(x,y,false))
			{
				gamedata.setScreenBuff(x,y,'O',false);
					return;		
			}
		}
	}
}
